package HalfMonthlyArchives::HalfMonthly;
use strict;
use base qw( MT::ArchiveType::Date );

use HalfMonthlyArchives::Util qw( ts2half start_end_half_month );

sub name {
    return 'HalfMonthly';
}

sub archive_label {
    my $plugin = MT::Plugin::HalfMonthlyArchives->instance;
    $plugin->translate('HALF-MONTHLY_ADV');
}

sub dynamic_template {
    return 'archives/<$MTArchiveDate format="%Y%m"$><$MTArchiveHalfMonth$>';
}

sub default_archive_templates {
    return [
        {
            label    => 'yyyy/mm/half/index.html',
            template => '%y/%m/<$MTArchiveHalfMonthLabel$>/%i',
            default  => 1,
        },
    ];
}

sub template_params {
    return {
        archive_class                  => "datebased-half-monthly-archive",
        datebased_only_archive         => 1,
        datebased_half_monthly_archive => 1,
        archive_template               => 1,
        archive_listing                => 1,
    };
}

sub archive_file {
    my $obj = shift;
    my ( $ctx, %param ) = @_;
    my $timestamp = $param{Timestamp};
    my $file_tmpl = $param{Template};
    my $blog      = $ctx->{__stash}{blog};

    my $file;
    if ($file_tmpl) {
        ( $ctx->{current_timestamp}, $ctx->{current_timestamp_end} ) =
          start_end_half_month($timestamp);
    }
    else {
        my $start = start_end_half_month( $timestamp, $blog );
        my ( $year, $mon, $mday ) = unpack 'A4A2A2', $start;
        $file = sprintf( "%04d/%02d/%02d/index", $year, $mon, $mday );
    }

    $file;
}

sub archive_title {
    my $obj = shift;
    my ( $ctx, $entry_or_ts ) = @_;
    my $stamp = ref $entry_or_ts ? $entry_or_ts->authored_on : $entry_or_ts;
    Carp::confess("ctx is undef") if !defined($ctx);
    my $start = start_end_half_month( $stamp, $ctx->stash('blog') );
    require MT::Template::Context;
    my $month = MT::Template::Context::_hdlr_date( $ctx,
        { ts => $start, 'format' => "%B %Y" } );
    my $hm = ts2half($start);

    my $lang = lc MT->current_language || 'en_us';
    $lang = 'ja' if lc($lang) eq 'jp';

    if ($hm == 1) {
        sprintf( "%s%s", $month, ( $lang eq 'ja' ? '&#19978;&#26092;' : ' First Half' ) );
    } else {
        sprintf( "%s%s", $month, ( $lang eq 'ja' ? '&#19979;&#26092;' : ' Last Half' ) );
    }
}

sub date_range {
    my $obj = shift;
    start_end_half_month(@_);
}

sub archive_group_iter {
    my $obj = shift;
    my ( $ctx, $args ) = @_;
    my $blog = $ctx->stash('blog');
    my $iter;
    my $sort_order =
      ( $args->{sort_order} || '' ) eq 'ascend' ? 'ascend' : 'descend';
    my $order = ( $sort_order eq 'ascend' ) ? 'asc' : 'desc';

    my $ts    = $ctx->{current_timestamp};
    my $tsend = $ctx->{current_timestamp_end};

    require MT::Entry;
    $iter = MT::Entry->count_group_by(
        {
            blog_id => $blog->id,
            status  => MT::Entry::RELEASE(),
            ( $ts && $tsend ? ( authored_on => [ $ts, $tsend ] ) : () ),
        },
        {
            ( $ts && $tsend ? ( range_incl => { authored_on => 1 } ) : () ),
            group => [
                "extract(year from authored_on)",
                "extract(month from authored_on)",
                "extract(day from authored_on)"
            ],
            $args->{lastn} ? ( limit => $args->{lastn} ) : (),
            sort => [
                { column => "extract(year from authored_on)", desc => $order },
                { column => "extract(month from authored_on)", desc => $order },
                { column => "extract(day from authored_on)", desc => $order }
            ],
        }
    ) or return $ctx->error("Couldn't get " . $obj->name . " archive list");

    return sub {
        while ( my @row = $iter->() ) {
            my $date = sprintf( "%04d%02d%02d000000", $row[1], $row[2], $row[3] );
            my ( $start, $end ) = start_end_half_month($date);
            return (
                $row[0],
                year  => $row[1],
                month => $row[2],
                day   => $row[3],
                start => $start,
                end   => $end
            );
        }
        undef;
    };
}

sub archive_group_entries {
    my $obj = shift;
    my ( $ctx, %param ) = @_;
    my $ts =
        $param{year}
    ? sprintf( "%04d%02d%02d000000", $param{year}, $param{month}, $param{start})
        : undef;
    my $limit = $param{limit};
    $obj->dated_group_entries( $ctx, 'HalfMonthly', $ts, $limit );
}

sub archive_entries_count {
    my $obj = shift;
    my ( $blog, $at, $entry ) = @_;
    return $obj->SUPER::archive_entries_count(
        {
            Blog        => $blog,
            ArchiveType => $at,
            Timestamp   => $entry->authored_on
        }
    );
}

1;
